/* MCVE v1.x C API
   (c) 2001 by Main Street Softworks, Inc.
   Written by Brad House

   This API is being released to the public domain to be modified and used in any manor the holder of this code sees fit.
   For any questions please contact  support@mainstreetsoftworks.com
*/

#ifndef __MCVE_H__
#define __MCVE_H__



/* #undef MCVE_PTHREAD */
#define MCVE_THREADSAFE 1
#define ENABLE_SSL 1
/* #undef NEED_SEED */
#ifdef MCVE_PTHREAD
#include <pthread.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define MCVE_FILE 1
#define MCVE_SOCKETS 2
#define MCVE_SSL 3

#define MCVE_SALE 1
#define MCVE_PREAUTH 2
#define MCVE_FORCE 3
#define MCVE_OVERRIDE 4
#define MCVE_RETURN 5
#define MCVE_VOID 6
#define MCVE_SETTLE 7
#define MCVE_CHKPWD 8
#define MCVE_ADDUSER 9
#define MCVE_DELUSER 10
#define MCVE_EDITUSER 11
#define MCVE_ENABLE 12
#define MCVE_DISABLE 13
#define MCVE_LISTUSERS 14
#define MCVE_GUT 15
#define MCVE_GL 16
#define MCVE_CHNGPWD 17
#define MCVE_UB 18
#define MCVE_GFT 19
#define MCVE_RS 20
#define MCVE_QC 21
#define MCVE_PBT 22
#define MCVE_BT 23
#define MCVE_LISTSTATS 24



// USER SETUP INFORMATION
#define MCVE_PROC 1
#define MCVE_USER 2
#define MCVE_PWD 3
#define MCVE_INDCODE 4
#define MCVE_MERCHID 5
#define MCVE_BANKID 6
#define MCVE_TERMID 7
#define MCVE_CLIENTNUM 8
#define MCVE_STOREID 9
#define MCVE_AGENTID 10
#define MCVE_CHAINID 11
#define MCVE_ZIPCODE 12
#define MCVE_TIMEZONE 13
#define MCVE_MERCHCAT 14
#define MCVE_MERNAME 15
#define MCVE_MERCHLOC 16
#define MCVE_STATECODE 17
#define MCVE_SERVICEPHONE 18


#define MCVE_LOCK_IN 1
#define MCVE_LOCK_OUT 2
#define MCVE_LOCK_AUTH 3

typedef struct MCVE_UserSetup {
  char *proc;
  char *user;
  char *pwd;
  char *indcode;
  char *merchid;
  char *bankid;
  char *termid;
  char *clientnum;
  char *storeid;
  char *agentid;
  char *chainid;
  char *zipcode;
  char *timezone;
  char *merchcat;
  char *mername;
  char *merchloc;
  char *statecode;
  char *servicephone;
} MCVE_UserSetup;

typedef struct MCVE_TRANS {
  int trans_type;
  int admin_type;
  char *username;
  char *password;
  char *account;
  char *trackdata;
  char *expdate;
  char *street;
  char *zip;
  char *cv;
  char *comments;
  char *clerkid;
  char *stationid;
  char *apprcode;
  double amount;
  long ptrannum;
  long sid;
  char *user;
  char *pwd;
  char *acct;
  char *bdate;
  char *edate;
  char *batch;
  int searchtype;
  int ping;
  MCVE_UserSetup  *usersetup;

} MCVE_TRANS;

#define MCVE_UNUSED 0
#define MCVE_PENDING 1
#define MCVE_DONE 2

#define MCVE_ERROR -1
#define MCVE_FAIL 0
#define MCVE_SUCCESS 1
#define MCVE_AUTH 2
#define MCVE_DENY 3
#define MCVE_CALL 4
#define MCVE_DUPL 5
#define MCVE_PKUP 6
#define MCVE_RETRY 7
#define MCVE_SETUP 8
#define MCVE_TIMEOUT 9

// AVS/CVV2 response codes
#define MCVE_GOOD 1
#define MCVE_BAD 0
#define MCVE_STREET 2
#define MCVE_ZIP 3
#define MCVE_UNKNOWN -1

typedef struct MCVE_QUEUE {
  char identifier[31];
  int status;

  int type;

  // Stuff read from return
  int code;
  int avs;
  int cv;
  char *auth;
  char *text;
  long tid;
  char *item;
  char *batch;

  // Comma Delimited response storage
  char *response;
  int iscommadelimiated;
    // storage
    char ***separated;
    int columns;
    int rows;
} MCVE_QUEUE;

typedef struct MCVE_CONN {
  int conn_method;  // Drop File, Sockets, SSL
  char location[251]; // Either Drop-File directory, or hostname
  unsigned long port;  // For IP and SSL, a port number
  int ptr;  // For IP and SSL, the pointer to an open connection
  char *inbuf;
  char *outbuf;
  long timeout;
  int verifyconn;
  int verifyssl;
#ifdef MCVE_THREADSAFE
#ifdef MCVE_PTHREAD
  pthread_mutex_t in_lock;
  pthread_mutex_t out_lock;
  pthread_mutex_t auth_lock;
#else
  int in_lock;
  int out_lock;
  int auth_lock;
#endif // MCVE_PTHREAD
#endif // MCVE_THREADSAFE

#ifdef ENABLE_SSL
  void *ssl;
  void *server_cert;
#endif
  // int CA_verify;
  // char *CA_cert;  // To store an SSL certificate
  // char *CA_key;  // To store an SSL key
  long outstanding_auths;
  long auth_buffer;
  MCVE_QUEUE *authorizations;
} MCVE_CONN;

#ifdef ENABLE_SSL
extern void *client_ctx;
#endif


/* BASE ROUTINES */
int MCVE_InitEngine(char *location);
void MCVE_InitConn(MCVE_CONN *conn);
void MCVE_DeleteResponse(MCVE_CONN *conn, long identifier);
void MCVE_DestroyConn(MCVE_CONN *conn);
int MCVE_SetTimeout(MCVE_CONN *conn, long timeout);
int MCVE_SetDropFile(MCVE_CONN *conn, char *df_location);
int MCVE_SetIP(MCVE_CONN *conn, char *host, unsigned long port);
int MCVE_SetSSL(MCVE_CONN *conn, char *host, unsigned long port);
int MCVE_Connect(MCVE_CONN *conn);
int MCVE_Monitor(MCVE_CONN *conn);
int MCVE_TransactionsSent(MCVE_CONN *conn);
void MCVE_DestroyEngine();
long MCVE_Ping(MCVE_CONN *conn);
void MCVE_VerifyConnection(MCVE_CONN *conn, int tf);
void MCVE_VerifySSLCert(MCVE_CONN *conn, int tf);


/* Transaction Routines */
long MCVE_Sale(MCVE_CONN *conn, char *username, char *password, char *trackdata, char *account, char *expdate, double amount,
                               char *street, char *zip, char *cv, char *comments, char *clerkid, char *stationid, long ptrannum);
long MCVE_PreAuth(MCVE_CONN *conn, char *username, char *password, char *trackdata, char *account, char *expdate, double amount,
                                     char *street, char *zip, char *cv, char *comments, char *clerkid, char *stationid, long ptrannum);
long MCVE_Void(MCVE_CONN *conn, char *username, char *password, long sid, long ptrannum);
long MCVE_PreAuthCompletion(MCVE_CONN *conn, char *username, char *password, double finalamount, long sid, long ptrannum);
long MCVE_Force(MCVE_CONN *conn, char *username, char *password, char *trackdata, char *account, char *expdate, double amount,
                char *authcode, char *comments, char *clerkid, char *stationid, long ptrannum);
long MCVE_Return(MCVE_CONN *conn, char *username, char *password, char *trackdata, char *account, char *expdate, double amount,
                                     char *comments, char *clerkid, char *stationid, long ptrannum);
long MCVE_Override(MCVE_CONN *conn, char *username, char *password, char *trackdata, char *account, char *expdate, double amount,
                                     char *street, char *zip, char *cv, char *comments, char *clerkid, char *stationid, long ptrannum);
long MCVE_Settle(MCVE_CONN *conn, char *username, char *password, char *batch);


/* Response Checking Routines */
int MCVE_ReturnStatus(MCVE_CONN *conn, long identifier);
int MCVE_ReturnCode(MCVE_CONN *conn, long identifier);
long MCVE_TransactionItem(MCVE_CONN *conn, long identifier);
long MCVE_TransactionBatch(MCVE_CONN *conn, long identifier);
long MCVE_TransactionID(MCVE_CONN *conn, long identifier);
char *MCVE_TransactionAuth(MCVE_CONN *conn, long identifier);
char *MCVE_TransactionText(MCVE_CONN *conn, long identifier);
int MCVE_TransactionAVS(MCVE_CONN *conn, long identifier);
int MCVE_TransactionCV(MCVE_CONN *conn, long identifier);
long MCVE_TransInQueue(MCVE_CONN *conn);
int MCVE_CheckStatus(MCVE_CONN *conn, long trans);
long MCVE_CompleteAuthorizations(MCVE_CONN *conn, long **listings);


/* User-level administrative requests */
long MCVE_Gut(MCVE_CONN *conn, char *username, char *password, int type, char *acct, char *clerkid, char *stationid, char *comments,
                              double ptrannum, char *bdate, char *edate);
long MCVE_Gl(MCVE_CONN *conn, char *username, char *password, int type, char *acct, char *batch, char *clerkid, char *stationid, char *comments,
                              double ptrannum, char *bdate, char *edate);
long MCVE_Gft(MCVE_CONN *conn, char *username, char *password, int type, char *acct, char *clerkid, char *stationid, char *comments,
                              double ptrannum, char *bdate, char *edate);
long MCVE_Ub(MCVE_CONN *conn, char *username, char *password);
long MCVE_Rs(MCVE_CONN *conn, char *username, char *password, char *stationid);
long MCVE_Qc(MCVE_CONN *conn, char *username, char *password, char *clerkid, char *stationid, char *comments, double ptrannum);
long MCVE_Chkpwd(MCVE_CONN *conn, char *username, char *password);
long MCVE_Bt(MCVE_CONN *conn, char *username, char *password);

/* Engine-Admin requests */
long MCVE_Chngpwd(MCVE_CONN *conn, char *adminpwd, char *newpassword);
long MCVE_ListUsers(MCVE_CONN *conn, char *adminpwd);
long MCVE_AddUser(MCVE_CONN *conn, char *adminpwd, MCVE_UserSetup *usersetup);
long MCVE_EnableUser(MCVE_CONN *conn, char *adminpwd, char *user);
long MCVE_DisableUser(MCVE_CONN * conn, char *adminpwd, char *user);
char *MCVE_GetUserArg(MCVE_UserSetup *usersetup, int argtype);
int MCVE_AddUserArg(MCVE_UserSetup *usersetup, int argtype, char *argval);
void MCVE_DeleteUserSetup(MCVE_UserSetup *usersetup);
void MCVE_InitUserSetup(MCVE_UserSetup *usersetup);
long MCVE_DelUser(MCVE_CONN *conn, char *adminpwd, char *user);
long MCVE_EditUser(MCVE_CONN *conn, char *adminpwd, MCVE_UserSetup *usersetup);
long MCVE_ListStats(MCVE_CONN *conn, char *adminpwd);
// long MCVE_GetUserInfo(MCVE_CONN *conn, char *adminpwd, char *user);
// MCVE_UserSetup*MCVE_ParseUserInfo(MCVE_CONN *conn, long identifier);


/* Comma Delimited Routines */
int MCVE_IsCommaDelimited(MCVE_CONN *conn, long identifier);
int MCVE_ParseCommaDelimited(MCVE_CONN *conn, long identifier);
char *MCVE_GetCommaDelimited(MCVE_CONN *conn, long identifier);
char *MCVE_GetCell(MCVE_CONN *conn, long identifier, char *column, long row);
char *MCVE_GetCellByNum(MCVE_CONN *conn, long identifier, int column, long row);
int MCVE_NumColumns(MCVE_CONN *conn, long identifier);
long MCVE_NumRows(MCVE_CONN *conn, long identifier);
char *MCVE_GetHeader(MCVE_CONN *conn, long identifier, int column_num);



#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
